/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/


#include "AcpiWidget.h"
#include "AcpiConfigurationDialog.h"

#include "BaseContextMenu.h"
#include "Color.h"
#include "CustomPixmap.h"
#include "Debug.h"
#include "GridLayout.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "ToolTipWidget.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QLayout>
#include <QPainter>
#include <QStringList>
#include <QTextStream>
#include <QRegExp>

namespace Acpi
{
    //__________________________________________________
    AcpiWidget::AcpiWidget( QWidget* parent ):
        MainWindow( parent )
    {

        Debug::Throw( "AcpiWidget::AcpiWidget.\n" );
        setWindowTitle( Util::windowTitle( tr( "Battery Monitor" ) ) );

        QHBoxLayout* hLayout( new QHBoxLayout );
        setLayout( hLayout );
        hLayout->setSpacing(0);
        hLayout->setMargin(0);

        // label
        hLayout->addWidget( label_ = new QLabel( this ), 0, Qt::AlignVCenter );

        // progressbar
        hLayout->addWidget( progressBar_ = new QProgressBar( this ), 1, Qt::AlignVCenter );
        progressBar_->setMaximum( 100 );

        // actions
        _installActions();
        _configurationAction().setText( tr( "Battery Monitor Settings..." ) );

        // connections
        connect( &acpi_, SIGNAL(updated()), SLOT(_update()) );

        _updateConfiguration();
        _toggleInverseColors( inverseColorsAction().isChecked() );

    }

    //________________________________________________
    void AcpiWidget::_configuration()
    {

        Debug::Throw( "AcpiWidget::_configuration.\n" );

        AcpiConfigurationDialog dialog(0);
        _configureFromDialog( dialog );

    }

    //__________________________________________________________
    void AcpiWidget::_updateConfiguration()
    {
        Debug::Throw( "AcpiWidget::_updateConfiguration.\n" );

        // color
        colors_[Acpi::Normal] = XmlOptions::get().get<Base::Color>( "NORMAL_COLOR" );
        colors_[Acpi::Low] = XmlOptions::get().get<Base::Color>( "LOW_COLOR" );
        colors_[Acpi::Critical] = XmlOptions::get().get<Base::Color>( "CRITICAL_COLOR" );

        acpi_.setThreshold( Acpi::Low, XmlOptions::get().get<int>( "LOW_THRESHOLD" ) );
        acpi_.setThreshold( Acpi::Critical, XmlOptions::get().get<int>( "CRITICAL_THRESHOLD" ) );

        // label
        _showLabelAction().setChecked( XmlOptions::get().get<bool>( "SHOW_LABEL" ) );

        // pixmap
        pixmaps_.clear();
        pixmaps_[Acpi::Charged] = CustomPixmap().find( IconNames::BatteryCharged ).scaled( IconSize::get( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );
        pixmaps_[Acpi::Charging] = CustomPixmap().find( IconNames::BatteryCharging ).scaled( IconSize::get( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );
        pixmaps_[Acpi::Discharging] = CustomPixmap().find( IconNames::BatteryDischarging ).scaled( IconSize::get( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );
        pixmaps_[Acpi::Unknown] = CustomPixmap().find( IconNames::BatteryUnknown ).scaled( IconSize::get( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );

        // parent class
        MainWindow::_updateConfiguration();

        // margins
        setContentsMargins( _margins().left(), _margins().top(), _margins().right(), _margins().bottom() );

        // adjust height to minimum
        resize( width(), minimumHeight() );

        _updateTimer().start( _refresh()*1000, this );
        acpi_.update();
        _update();

    }

    //______________________________________________________________________
    void AcpiWidget::_paint( QPaintDevice& device, const QRect& clip )
    {
        if( label_->isVisible() ) _render( device, *label_, clip );
        if( progressBar_->isVisible() ) _render( device, *progressBar_, clip );
    }

    //______________________________________________________________________
    void AcpiWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "AcpiWidget::_installContextMenuActions.\n" );

        // create parent menu
        MainWindow::_installContextMenuActions( menu );

        // actions
        menu->insertSeparator( &_menuSeparator() );
        menu->insertAction( &_menuSeparator(), &_showLabelAction() );
        menu->insertAction( &_menuSeparator(), &_updateAction() );

    }

    //__________________________________________________________
    BaseToolTipWidget* AcpiWidget::_createToolTipWidget()
    {
        ToolTipWidget* widget = new ToolTipWidget( this );
        widget->setPixmap( windowIcon().pixmap( IconSize::get( IconSize::Maximum ) ) );

        QLabel* label;
        toolTipItems_.append( label = new QLabel( widget ) );
        label->setAlignment( Qt::AlignCenter );
        widget->layout().addWidget( label );

        toolTipItems_.append( label = new QLabel( widget ) );
        label->setAlignment( Qt::AlignCenter );
        widget->layout().addWidget( label );

        return widget;
    }

    //__________________________________________________
    void AcpiWidget::timerEvent( QTimerEvent *event )
    {

        if( event->timerId() == _updateTimer().timerId() ) acpi_.update();
        MainWindow::timerEvent( event );

    }

    //______________________________________________________________________
    void AcpiWidget::_toggleLabel( bool state )
    {
        Debug::Throw( "AcpiWidget::_toggleLabel.\n" );
        label_->setVisible( state );
        XmlOptions::get().set<bool>( "SHOW_LABEL", state );
    }


    //____________________________________________________________________
    void AcpiWidget::_toggleInverseColors( bool value )
    {

        Debug::Throw( "AcpiWidget::_toggleInverseColors.\n" );

        if( progressBar_ )
        {

            auto palette( this->palette() );
            if( value )
            {
                auto&& tmp = palette.color( QPalette::Window );
                palette.setColor( QPalette::Window, palette.color( QPalette::WindowText ) );
                palette.setColor( QPalette::WindowText, tmp );
            }

            progressBar_->setPalette( palette );
            _update();
        }

        TransparentWindow::_toggleInverseColors( value );
        update();

    }

    //______________________________________________________________________
    bool AcpiWidget::_updateToolTip()
    {

        Debug::Throw( "AcpiWidget::_updateToolTip.\n" );

        // title
        static_cast<ToolTipWidget*>( _toolTipWidget() )->setTitle( windowTitle() );

        if( !acpi_.stateText().isEmpty() )
        {
            toolTipItems_[0]->show();
            toolTipItems_[0]->setText( QString( tr( "Status: %1" ) ).arg( acpi_.stateText() ) );

        } else toolTipItems_[0]->hide();

        if( !acpi_.tooltipText().isEmpty() )
        {
            toolTipItems_[1]->show();
            toolTipItems_[1]->setText( acpi_.tooltipText() );
        } else toolTipItems_[1]->hide();

        return _toolTipWidget()->isHidden();

    }

    //__________________________________________________
    void AcpiWidget::_installActions()
    {

        Debug::Throw( "AcpiWidget::_installActions.\n" );
        addAction( updateAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Update" ), this ) );
        _updateAction().setShortcut( Qt::CTRL + Qt::Key_U );
        _updateAction().setToolTip( tr( "Update" ) );
        connect( &_updateAction(), SIGNAL(triggered()), &acpi_, SLOT(update()));

        addAction( showLabelAction_ = new QAction( tr( "Show Status" ), this ) );
        _showLabelAction().setToolTip( tr( "Show/hide status label" ) );
        _showLabelAction().setCheckable( true );
        _showLabelAction().setChecked( true );
        connect( &_showLabelAction(), SIGNAL(toggled(bool)), SLOT(_toggleLabel(bool)));

    }

    //________________________________________________
    void AcpiWidget::_render( QPaintDevice& device, QWidget& widget, const QRect& clip )
    { widget.render( &device, widget.geometry().topLeft(), widget.rect().intersected( clip.translated( -widget.geometry().topLeft() ) ), 0 ); }

    //_________________________________________________________
    void AcpiWidget::_update()
    {

        Debug::Throw( "AcpiWidget::_update.\n" );

        auto palette( progressBar_->palette() );

        auto&& color( colors_[acpi_.level()] );
        if( color.isValid() ) palette.setColor( QPalette::Highlight, color );
        progressBar_->setPalette( palette );

        label_->setPixmap( pixmaps_[acpi_.state()] );
        label_->update();

        progressBar_->setValue( acpi_.progress() );

        update();

        return;

    }

}
