#ifndef CalendarCell_h
#define CalendarCell_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"

#include <QDate>
#include <QRectF>
#include <QList>
#include <algorithm>

namespace Calendar
{
    //* store cells (date and rect
    class Cell final: private Base::Counter<Cell>
    {

        public:

        //* constructor
        explicit Cell():
            Counter( "Calendar::Cell" ),
            enabled_( false )
        {}

        //* constructor
        explicit Cell( QDate date, QRectF rect );

        //*@name accessors
        //@{

        //* valid
        bool isValid() const
        { return date_.isValid() && !rect_.isNull(); }

        //* date
        const QDate& date() const
        { return date_; }

        //* rect
        const QRectF& rect() const
        { return rect_; }

        //* rect


        //* enabled
        bool isEnabled() const
        { return enabled_; }

        //@}

        //* enabled
        void setEnabled( bool value )
        { enabled_ = value; }

        //* used to find cell that match a given point
        class ContainsFTor
        {
            public:

            //* constructor
            explicit ContainsFTor( const QPoint& point ):
                point_( point )
             {}

            //* predicate
            bool operator() (const Cell& cell ) const
            { return cell.rect().contains( point_ ); }

            private:

            //* prediction
            QPoint point_;

        };

        using List = QList<Cell>;

        //* find date matching point
        static const Cell findCell( const List& list, const QPoint& position )
        {
            auto iter = std::find_if( list.begin(), list.end(), ContainsFTor( position ) );
            return iter == list.end() ? Cell():*iter;
        }

        //* find cells matching given week
        static const List selectWeek( const List& list, const QDate& date )
        {
            if( !date.isValid() ) return List();
            List out;

            for( const auto& cell:list )
            { if( cell.date().weekNumber() == date.weekNumber() ) out << cell; }

            return out;
        }

        //* find cells matching given week day
        static const List selectWeekDay( const List& list, const QDate& date )
        {
            if( !date.isValid() ) return List();
            List out;
            for( const auto& cell:list )
            { if( cell.date().dayOfWeek() == date.dayOfWeek() ) out << cell; }

            return out;
        }

        private:

        //* date
        QDate date_;

        //* active
        bool enabled_;

        //*rect
        QRectF rect_;

    };

}

#endif
