/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CalendarEventDialog.h"

#include "CalendarEventDelegate.h"
#include "CalendarEventList.h"
#include "EditCalendarEventDialog.h"
#include "FileDialog.h"
#include "File.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "InformationDialog.h"
#include "QtUtil.h"
#include "TreeView.h"
#include "XmlOptions.h"

#include <QHeaderView>

namespace Calendar
{

    //________________________________________________________
    CalendarEventDialog::CalendarEventDialog( QWidget* parent, const CalendarEvent::List& list ):
        CustomDialog( parent )
    {

        Debug::Throw( "CalendarEventDialog::CalendarEventDialog.\n" );

        setOptionName( "CALENDAR_EVENT_DIALOG" );
        setWindowTitle( tr( "Events - Calendar" ) );

        QHBoxLayout* layout( new QHBoxLayout );
        layout->setSpacing(5);
        layout->setMargin(0);
        mainLayout().addLayout( layout );

        // create list
        list_ = new TreeView( this );
        list_->setModel( &model_ );
        list_->setSelectionMode( QAbstractItemView::ContiguousSelection );
        list_->setSortingEnabled( false );

        // replace item delegate
        if( list_->itemDelegate() ) list_->itemDelegate()->deleteLater();
        list_->setItemDelegate( new CalendarEventDelegate( this ) );

        list_->setMask( 1 );
        list_->forceAlternatingRowColors();
        list_->header()->hide();

        layout->addWidget( list_, 1 );

        // set connections
        connect( list_, SIGNAL(activated(QModelIndex)), SLOT(_edit()) );
        connect( list_->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );

        // buttons
        QVBoxLayout* buttonLayout = new QVBoxLayout;
        buttonLayout->setMargin(0);
        buttonLayout->setSpacing( 5 );
        layout->addLayout( buttonLayout, 0 );

        // Add
        QPushButton *button;
        buttonLayout->addWidget( button = new QPushButton( IconEngine::get( IconNames::Add ), tr( "Add" ), this ) );
        connect( button, SIGNAL(clicked()), SLOT(_add()) );

        // Edit
        buttonLayout->addWidget( edit_ = new QPushButton( IconEngine::get( IconNames::Edit ), tr( "Edit" ), this ) );
        connect( edit_, SIGNAL(clicked()), SLOT(_edit()) );
        edit_->setToolTip( tr( "Edit selected alarm" ) );

        // remove
        addAction( removeAction_ = new QAction( "Remove", this ) );
        connect( removeAction_, SIGNAL(triggered()), SLOT(_remove()) );
        removeAction_->setShortcut( QKeySequence::Delete );
        removeAction_->setToolTip( tr( "Remove selected alarm" ) );

        // remove
        buttonLayout->addWidget( remove_ = new QPushButton( IconEngine::get( IconNames::Remove ), tr( "Remove" ), this ) );
        connect( remove_, SIGNAL(clicked()), SLOT(_remove()) );
        remove_->setToolTip( tr( "Remove selected alarm" ) );

        // separator
        QFrame* frame( new QFrame( this ) );
        frame->setFrameStyle( QFrame::HLine );
        buttonLayout->addWidget( frame );

        // import
        buttonLayout->addWidget( button = new QPushButton( IconEngine::get( IconNames::Import ), tr( "Import" ), this ) );
        connect( button, SIGNAL(clicked()), SLOT(_import()) );
        button->setToolTip( tr( "Import events from external file" ) );

        // export
        buttonLayout->addWidget( export_ = new QPushButton( IconEngine::get( IconNames::Export ), tr( "Export" ), this ) );
        connect( export_, SIGNAL(clicked()), SLOT(_export()) );
        export_->setToolTip( tr( "Export events to external file" ) );

        buttonLayout->addStretch(1);

        // update buttons
        _updateButtons();

        // set list
        setList( list );

        // ensure last item is visible
        if( !list.isEmpty() )
        { list_->scrollTo( model_.index( model_.rowCount()-1, 0 ) ); }

    }

    //_____________________________________________________________________
    void CalendarEventDialog::setList( const CalendarEvent::List& list )
    {
        Debug::Throw( "CalendarEventDialog::setList.\n" );
        model_.update( list );
        list_->resizeColumns();
        export_->setEnabled( !list.isEmpty() );
    }

    //_____________________________________________________________________
    CalendarEvent::List CalendarEventDialog::list() const
    {
        Debug::Throw( "CalendarEventDialog::list.\n" );
        return model_.get();
    }

    //_________________________________________________________________________
    void CalendarEventDialog::_updateButtons()
    {
        Debug::Throw( "CalendarEventDialog::_updateButtons.\n" );
        int selection_size( list_->selectionModel()->selectedRows().size() );

        // enable buttons depending on the size of the list
        edit_->setEnabled( selection_size == 1 );
        removeAction_->setEnabled( selection_size != 0 );
        remove_->setEnabled( selection_size != 0 );

    }

    //_________________________________________________________________________
    void CalendarEventDialog::_add()
    {
        Debug::Throw( "CalendarEventDialog::_add.\n" );

        CalendarEvent current;
        current.setRange( QDate::currentDate() );
        EditCalendarEventDialog dialog( this, current );
        if( dialog.centerOnWidget( window() ).exec() == QDialog::Rejected ) return;

        // check event and add to model
        auto event( dialog.event() );
        auto error( event.validate() );
        if( !error.isEmpty() ) InformationDialog( this, error ).centerOnWidget( window() ).exec();

        model_.add( event );

        list_->resizeColumns();
        export_->setEnabled( model_.rowCount() > 0 );
        return;
    }

    //_________________________________________________________________________
    void CalendarEventDialog::_edit()
    {
        Debug::Throw( "CalendarEventDialog::_edit.\n" );

        // retrieve selection
        QModelIndex current( list_->selectionModel()->currentIndex() );
        Q_ASSERT( current.isValid() );

        EditCalendarEventDialog dialog( this, model_.get( current ) );
        if( dialog.centerOnWidget( window() ).exec() == QDialog::Rejected ) return;

        model_.remove( model_.get( current ) );

        // check event and add to model
        CalendarEvent event( dialog.event() );

        QString error( event.validate() );
        if( !error.isEmpty() ) InformationDialog( this, error ).centerOnWidget( window() ).exec();

        model_.add( event );
        export_->setEnabled( model_.rowCount() > 0 );

        list_->resizeColumns();

        return;

    }

    //__________________________________________________
    void CalendarEventDialog::_remove()
    {
        Debug::Throw( "CalendarEventDialog::_remove.\n" );

        // retrieve selected items; make sure they do not include the navigator
        model_.remove( model_.get( list_->selectionModel()->selectedRows() ) );
        list_->resizeColumns();
        export_->setEnabled( model_.rowCount() > 0 );

    }

    //__________________________________________________
    void CalendarEventDialog::_import()
    {
        Debug::Throw( "CalendarEventDialog::_import.\n" );

        // create file dialog
        FileDialog dialog( this );
        dialog.setFileMode( QFileDialog::ExistingFile );
        if( XmlOptions::get().contains( "EXPORT_FILE" ) )
        {
            const File defaultFile( XmlOptions::get().raw( "EXPORT_FILE" ) );
            if( defaultFile.exists() ) dialog.selectFile( defaultFile );
        }


        File file( dialog.getFile() );
        if( file.isEmpty() ) return;

        XmlOptions::get().set( "EXPORT_FILE", file.expanded() );

        CalendarEventList list;
        if( !list.read( file ) )
        {
            const QString what = QString( tr( "Failed to read calendar events from file '%1'" ) ).arg( file );
            InformationDialog( this, what ).exec();
            return;

        } else {

            model_.add( list.events() );

        }

    }

    //__________________________________________________
    void CalendarEventDialog::_export()
    {
        Debug::Throw( "CalendarEventDialog::_export.\n" );

        // create file dialog
        FileDialog dialog( this );
        dialog.setFileMode( QFileDialog::AnyFile );
        dialog.setAcceptMode( QFileDialog::AcceptSave );

        // set default file
        if( XmlOptions::get().contains( "EXPORT_FILE" ) )
        {
            const File defaultFile( XmlOptions::get().raw( "EXPORT_FILE" ) );
            dialog.selectFile( defaultFile );
        }

        File file( dialog.getFile() );
        if( file.isEmpty() ) return;
        else file.expand();

        XmlOptions::get().set( "EXPORT_FILE", file );

        //! write to file
        if( !CalendarEventList( list() ).write( file ) )
        {
            const QString what = QString( tr( "Failed to write calendar events to file '%1'" ) ).arg( file );
            InformationDialog( this, what ).exec();
        }

        return;

    }

}
