#ifndef CalendarWidget_h
#define CalendarWidget_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CalendarCell.h"
#include "CalendarPath.h"
#include "CalendarEventList.h"
#include "MainWindow.h"

#include <QAbstractItemView>
#include <QBasicTimer>
#include <QDateTime>
#include <QTimerEvent>
#include <QPolygonF>
#include <QAction>

//* Calendar widgets
namespace Calendar
{

    //* forward declaration
    class CalendarHeader;

    //* transparent Calendar widget
    class CalendarWidget: public MainWindow
    {

        //* Qt meta object declaration
        Q_OBJECT

        public:

        //* constructor
        explicit CalendarWidget( QWidget* = nullptr );

        protected Q_SLOTS:

        //* header colors
        void _updateHeaderColors();

        protected:

        //*@name event handlers
        //@{

        //* mouse press event [overloaded]
        void mousePressEvent( QMouseEvent* ) override;

        //* mouse move event
        /*! used to track date matching pointer */
        void mouseMoveEvent( QMouseEvent* ) override;

        //* context menu event
        void contextMenuEvent( QContextMenuEvent* ) override;

        //* paint event handler
        void _paint( QPaintDevice&, const QRect& ) override;

        //* timer event
        void timerEvent( QTimerEvent* ) override;

        //@}

        //* context menu
        void _installContextMenuActions( BaseContextMenu* ) override;

        //* tooltip widget
        BaseToolTipWidget* _createToolTipWidget() override;

        //*@name current events
        //@{

        //* store list of current month events
        void _updateCurrentEvents();

        //* current events
        const CalendarEvent::List& _currentEvents() const
        { return currentEvents_; }

        //* current path list
        void _setCurrentPathList( const Path::List& value )
        { currentPath_ = value; }

        //* current path list
        const Path::List& _currentPathList() const
        { return currentPath_; }

        //* update current path list
        void _updateCurrentPathList()
        { _setCurrentPathList( _pathList( _currentEvents() ) ); }

        //* paint current path list
        void _paintCurrentPathList( QPainter& painter )
        { _paintPathList( painter, _currentPathList() ); }

        //@}

        //*@name selected events
        //@{

        //* update selection to include given position
        void _updateSelection( const QPoint&, CalendarEvent::SelectionMode = CalendarEvent::SelectionRange );

        //* store list of selected month events
        void _updateSelectedDays( const QDate&, const CalendarEvent::SelectionMode&, bool = false );

        //* clear selected events
        void _clearSelectedDays()
        { _updateSelectedDays( QDate(), CalendarEvent::SelectionRange, true ); }

        //* selected events
        const CalendarEvent::List& _selectedDays() const
        { return selectedDays_; }

        //* selected date
        void _setSelectedDate( const QDate& value )
        { selectedData_ = value; }

        //* selected date
        const QDate& _selectedDate() const
        { return selectedData_; }

        //* selected path list
        void _setSelectedPathList( const Path::List& value )
        { selectedPathList_ = value; }

        //* selected path list
        const Path::List& _selectedPathList() const
        { return selectedPathList_; }

        //* update selected path list
        void _updateSelectedPathList()
        { _setSelectedPathList( _pathList( _selectedDays() ) ); }

        //* paint selected path list
        void _paintSelectedPathList( QPainter& painter )
        { _paintPathList( painter, _selectedPathList() ); }

        //@}


        //*@name geometry
        //@{

        //* update geometry
        void _updateGeometry();

        //* rows
        int _nRows() const
        { return nRows_; }

        //* columns
        int _nColumns() const
        { return nColumns_; }

        //* cell width
        const qreal& _cellWidth() const
        { return cellWidth_; }

        //* cell height
        const qreal& _cellHeight() const
        { return cellHeight_; }

        //* cell size
        const QSizeF _cellSize() const
        { return QSizeF( cellWidth_, cellHeight_ ); }

        //@}

        //* get path list associated to events
        Path::List _pathList( const CalendarEvent::List& ) const;

        //* paint events
        void _paintPathList( QPainter&, const Path::List& ) const;

        //* paint horizontal line
        void _paintSeparator( QPainter& ) const;

        //* paint text
        void _paintText( QPainter& ) const;

        //* current date
        bool _setDate( const QDate& );

        //* restart timer for auto-update of current day
        void _restartTimer();

        //* update widget tooltip, if any
        virtual bool _updateToolTip( const QPoint& );

        //* notifications
        void _checkNotifications();

        protected Q_SLOTS:

        //* configuration
        void _configuration() override;

        //* update configuration
        void _updateConfiguration() override;

        //* adjust time
        virtual void _adjustTime();

        //* previous month
        virtual void _selectPreviousMonth()
        {
            if( _setDate( date_.addMonths( -1 ) ) )
            { update(); }
        }

        //* previous year
        virtual void _selectPreviousYear()
        {
            if( _setDate( date_.addYears( -1 ) ) )
            { update(); }
        }

        //* next month
        virtual void _selectNextMonth()
        {
            if( _setDate( date_.addMonths( 1 ) ) )
            { update(); }
        }

        //* next year
        virtual void _selectNextYear()
        {
            if( _setDate( date_.addYears( 1 ) ) )
            { update(); }
        }

        //* show current month
        void _selectCurrentMonth()
        {
            if( _setDate( QDate::currentDate() ) )
            { update(); }
        }

        //* select month from dialog
        virtual void _selectMonthFromDialog();

        //* edit event
        virtual void _editEvent();

        //* remove event
        virtual void _removeEvent();

        //* event selection
        virtual void _addEvent();

        //* configure events
        virtual void _configureEvents();

        //* toggle showing of past events
        virtual void _toggleShowPastEvents( bool );

        //* toggle showing of week numbers
        virtual void _toggleShowWeekNumbers( bool );

        //* toggle showing of week numbers
        virtual void _toggleShowInvalidDays( bool );

        //* event notification sent
        virtual void _eventNotificationSent( const CalendarEvent& );

        private:

        //* install actions
        void _installActions();

        //* some styles require an item view passed to painting method to have proper selection rendered in items
        QAbstractItemView* itemView_ = nullptr;

        //*@name actions
        //@{

        //* previous month
        QAction* previousMonthAction_ = nullptr;

        //* previous year
        QAction* previousYearAction_ = nullptr;

        //* next month
        QAction* nextMonthAction_ = nullptr;

        //* next year
        QAction* nextYearAction_ = nullptr;

        //* current month action
        QAction* currentMonthAction_ = nullptr;

        //* select action
        /* used to select month, year by date */
        QAction* selectMonthAction_ = nullptr;

        //* edit event
        QAction* editEventAction_ = nullptr;

        //* remove event
        QAction* removeEventAction_ = nullptr;

        //* event selection
        QAction* addEventAction_ = nullptr;

        //* add event
        QAction* configureEventsAction_ = nullptr;

        //* adjust time
        QAction* adjustTimeAction_ = nullptr;

        //* past events
        QAction* showPastEventsAction_ = nullptr;

        //* week numbers
        QAction* showWeekNumbersAction_ = nullptr;

        //* invalid days
        QAction* showInvalidDaysAction_ = nullptr;

        //@}

        //* foreground color
        QColor currentDayColor_;

        //*@name geometry
        //@{

        //* columns
        int nColumns_ = 7;

        //* rows
        int nRows_ = 7;

        //* cell width
        qreal cellWidth_ = 0;

        //* cell height
        qreal cellHeight_ = 0;

        //* cells
        Cell::List cells_;

        //* week cells
        Cell::List weekCells_;

        //* week days
        Cell::List weekDayCells_;

        //* used to sort rectangles
        class SortRectFTor
        {
            public:

            bool operator() ( const QRectF& first, const QRectF& second ) const
            {
                return
                    first.top() < second.top() ||
                    ( first.top() == second.top() && first.left() < second.left() );
            }

        };

        //@}

        //* redraw timer
        QBasicTimer timer_;

        //* header
        CalendarHeader* header_ = nullptr;

        //* calendar event list
        CalendarEventList events_;

        //* calendar current event list
        CalendarEvent::List currentEvents_;

        //* current path list
        Path::List currentPath_;

        //* current event, if any
        CalendarEvent currentEvent_;

        //* selected date
        QDate selectedData_;

        //* calendate selected events
        CalendarEvent::List selectedDays_;

        //* current event, if any
        CalendarEvent selectedEvent_;

        //* selected path list
        Path::List selectedPathList_;

        //* displayed date
        QDate date_;

    };

};

#endif
