/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AdjustTimeDialog.h"
#include "ClockConfigurationDialog.h"

#include "ClockWidget.h"

#include "AlarmDialog.h"
#include "AlarmListDialog.h"
#include "BaseContextMenu.h"
#include "Color.h"
#include "Debug.h"
#include "Effects.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "QtUtil.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QDateTime>
#include <QProcess>

#include <QApplication>
#include <QPainter>

namespace Clock
{

    //_______________________________________________
    ClockWidget::ClockWidget( QWidget* parent ):
        MainWindow( parent ),
        delay_( 0 ),
        alarms_( this )
    {

        Debug::Throw( "Clock::ClockWidget::ClockWidget.\n" );
        setWindowTitle( Util::windowTitle( tr( "Transparent Clock" ) ) );

        // actions
        _installActions();
        _configurationAction().setText( tr( "Transparent Clock Settings..." ) );

        //* update configuration
        _updateConfiguration();

        //* read alarms
        alarms_.read( File( XmlOptions::get().raw( "RC_FILE" ) ) );
        if( !alarms_.alarms().empty() ) alarmTimer_.start( 1000, this );

        // start timer. update every second
        timer_.start( 1000, this );

        // write alarms at exit
        connect( qApp, SIGNAL(aboutToQuit()), &alarms_, SLOT(write()) );

    }

    //______________________________________________________________________
    void ClockWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "Clock::ClockWidget::_installContextMenuActions.\n" );

        // get menu from parent
        MainWindow::_installContextMenuActions( menu );

        // customize
        menu->insertSeparator( &_menuSeparator() );
        menu->insertAction( &_menuSeparator(), showHourTicksAction_ );
        menu->insertAction( &_menuSeparator(), showMinuteTicksAction_ );
        menu->insertAction( &_menuSeparator(), showSecondsAction_ );
        menu->insertAction( &_menuSeparator(), showDateAction_ );
        menu->insertSeparator( &_menuSeparator() );

        menu->insertAction( &_menuSeparator(), configureAlarmsAction_ );
        menu->insertAction( &_menuSeparator(), adjustTimeAction_ );

    }

    //______________________________________________________________________
    void ClockWidget::enterEvent( QEvent* event )
    {
        update();
        MainWindow::enterEvent( event );
    }

    //______________________________________________________________________
    void ClockWidget::timerEvent( QTimerEvent* event )
    {

        if( event->timerId() == timer_.timerId() ) update();
        else if( event->timerId() == alarmTimer_.timerId() ) _checkAlarms();
        else MainWindow::timerEvent( event );
        return;

    }

    //______________________________________________________________________
    void ClockWidget::_installActions()
    {

        Debug::Throw( "Clock::ClockWidget::_installActions.\n" );

        // alarms
        addAction( configureAlarmsAction_ = new QAction( IconEngine::get( IconNames::Alarm ), tr( "Configure Alarms..." ), this ) );
        configureAlarmsAction_->setToolTip( tr( "Add, remove or modify alarms" ) );
        connect( configureAlarmsAction_, SIGNAL(triggered()), SLOT(_configureAlarms()) );

        addAction( showHourTicksAction_ = new QAction( tr( "Show Ticks (hours)" ), this ) );
        showHourTicksAction_->setCheckable( true );
        showHourTicksAction_->setChecked( false );
        connect( showHourTicksAction_, SIGNAL(toggled(bool)), SLOT(_toggleHourTicks(bool)) );

        addAction( showMinuteTicksAction_ = new QAction( tr( "Show Ticks (minutes)" ), this ) );
        showMinuteTicksAction_->setCheckable( true );
        showMinuteTicksAction_->setChecked( false );
        connect( showMinuteTicksAction_, SIGNAL(toggled(bool)), SLOT(_toggleMinuteTicks(bool)) );

        addAction( showSecondsAction_ = new QAction( tr( "Show Seconds" ), this ) );
        showSecondsAction_->setCheckable( true );
        showSecondsAction_->setChecked( false );
        connect( showSecondsAction_, SIGNAL(toggled(bool)), SLOT(_toggleSeconds(bool)) );

        addAction( showDateAction_ = new QAction( tr( "Show Date" ), this ) );
        showDateAction_->setCheckable( true );
        showDateAction_->setChecked( false );
        connect( showDateAction_, SIGNAL(toggled(bool)), SLOT(_toggleDate(bool)) );

        addAction( adjustTimeAction_ = new QAction( tr( "Adjust Time..." ), this ) );
        adjustTimeAction_->setToolTip( tr( "Run external command to adjust system date and time" ) );
        connect( adjustTimeAction_, SIGNAL(triggered()), SLOT(_adjustTime()) );


    }

    //______________________________________________________________________
    void ClockWidget::_paint( QPaintDevice& device, const QRect& clip )
    {

        // get current time
        QTime time( QTime::currentTime() );
        QDate date( QDate::currentDate() );
        int hour( time.hour() );
        if( delay_ )
        {
            hour += delay_;
            while( hour < 0 ) { hour += 24; date = date.addDays( -1 ); }
            while( hour >= 24 ) { hour -= 24; date = date.addDays( 1 ); }
        }

        QPixmap pixmap( size() );
        pixmap.fill( Qt::transparent );

        {

            // prepare painter
            static const double virtualSide = 1000;
            QPainter painter( &pixmap );
            painter.setClipRect( clip );
            painter.setRenderHints(QPainter::Antialiasing );

            QRect rect( _margins().adjustedRect( this->rect() ) );
            if( showDateAction_->isChecked() )
            {
                // label
                const int labelHeight( fontMetrics().height() * 1.8 );
                const QRect textRect( rect.left(), rect.bottom() - labelHeight, rect.width(), labelHeight );
                rect.setBottom( textRect.top() );

                // retrieve current date
                painter.save();
                painter.setPen(  foregroundColor() );

                QString buffer;
                QTextStream( &buffer ) << date.day() << "/" << date.month() << "/" << date.year();
                painter.drawText( textRect, Qt::AlignHCenter|Qt::AlignBottom, buffer );
                painter.restore();

            }

            // translate painter
            painter.translate( rect.center() );

            static const int arrowWidth( 30 );
            int side = qMin( rect.width(), rect.height() ) ;
            painter.scale(side/virtualSide, side/virtualSide );

            // draw hours ticks
            if( showHourTicksAction_->isChecked() )
            {

                painter.save();
                painter.setPen( Qt::NoPen );
                painter.setBrush( hourColor_ );
                for (int j = 0; j < 12; ++j)
                {
                    if( j%3 == 0 ) painter.drawEllipse( QPointF(485, 0 ), 30, 30 );
                    painter.rotate(30.0);
                }
                painter.restore();

            }

            // draw minutes ticks
            if( showMinuteTicksAction_->isChecked() )
            {

                // draw minutes ticks
                painter.save();
                painter.setPen( minuteColor_ );
                for (int j = 0; j < 60; ++j)
                {
                    if ((j % 5) != 0) painter.drawPoint(500, 0);
                    painter.rotate(6.0);
                }
                painter.restore();

            }

            // draw center
            painter.setPen( Qt::NoPen );
            painter.save();
            painter.setBrush( hourColor_ );

            painter.drawEllipse( QPointF(0,0), arrowWidth, arrowWidth );

            // draw hours
            painter.rotate(30.0 * ( (hour + time.minute() / 60.0)));

            static const auto hourHand( _hand( arrowWidth, 400 ) );
            painter.drawPath( hourHand );
            painter.restore();

            // draw minutes
            painter.save();
            painter.setBrush( minuteColor_ );
            painter.rotate(6.0 * (time.minute() + time.second() / 60.0));

            static const auto minuteHand( _hand( arrowWidth, 450 ) );
            painter.drawPath( minuteHand );
            painter.restore();

            // draw seconds
            if( showSecondsAction_->isChecked() )
            {

                painter.save();
                painter.setBrush( secondColor_ );
                painter.rotate( 6.0*time.second() );

                static const auto secondHand( _hand( arrowWidth, 450 ) );
                painter.drawPath( secondHand );
                painter.restore();

            }

            painter.end();

        }

        if( !showSecondsAction_->isChecked() )
        {

            /*
            if seconds are not paint,
            adjust timer delay so that next refresh
            will be just after next minute change
            */
            timer_.start( 1000*(60-time.second() ) , this );

        }

        QPainter painter( &device );
        painter.setClipRect( clip );
        painter.setRenderHints(QPainter::Antialiasing );

        if( shadowOffset() > 0 && shadowColor().isValid() )
        {
            QImage image( pixmap.toImage() );
            Transparency::Effects::shadowBlur(image, shadowOffset(), shadowColor() );
            painter.drawImage(rect().topLeft(), image);
        }

        painter.drawPixmap( rect().topLeft(), pixmap );

        return;

    }

    //________________________________________________
    void ClockWidget::_setRepeat( const Alarm& alarm )
    {
        Debug::Throw( "Clock::ClockWidget::_setRepeat.\n" );
        for( auto& alarmIter:alarms_.alarms() )
        { if( alarmIter == alarm ) alarmIter.setRepeat( true ); }
    }

    //________________________________________________
    void ClockWidget::_unsetRepeat( const Alarm& alarm )
    {
        Debug::Throw( 0, "Clock::ClockWidget::_unsetRepeat.\n" );
        for( auto& alarmIter:alarms_.alarms() )
        { if( alarmIter == alarm ) alarmIter.setRepeat( false ); }
    }

    //________________________________________________
    void ClockWidget::_configuration()
    {

        Debug::Throw( "Clock::ClockWidget::_configuration.\n" );
        ClockConfigurationDialog dialog;
        _configureFromDialog( dialog );

    }

    //________________________________________________
    void ClockWidget::_updateConfiguration()
    {
        Debug::Throw( "Clock::ClockWidget::_updateConfiguration.\n" );

        // colors
        QColor color;
        bool useThemedColors( !XmlOptions::get().get<bool>( "USE_DEFAULT_COLORS" ) );
        _setHourColor( useThemedColors && (color = XmlOptions::get().get<Base::Color>( "HOUR_COLOR" ) ).isValid() ? color:foregroundColor() );
        _setMinuteColor( useThemedColors && (color = XmlOptions::get().get<Base::Color>( "MINUTE_COLOR" ) ).isValid() ? color:foregroundColor() );
        _setSecondColor( useThemedColors && (color = XmlOptions::get().get<Base::Color>( "SECOND_COLOR" ) ).isValid() ? color:foregroundColor() );

        // intensity
        hourColor_.setAlpha( foregroundIntensity() );
        minuteColor_.setAlpha( foregroundIntensity() );
        secondColor_.setAlpha( foregroundIntensity() );

        // delay
        delay_ = XmlOptions::get().get<int>( "CLOCK_DELAY" );

        // flags
        showHourTicksAction_->setChecked( XmlOptions::get().get<bool>( "SHOW_HOUR_TICKS" ) );
        showMinuteTicksAction_->setChecked( XmlOptions::get().get<bool>( "SHOW_MINUTE_TICKS" ) );
        showSecondsAction_->setChecked( XmlOptions::get().get<bool>( "SHOW_SECONDS" ) );
        showDateAction_->setChecked( XmlOptions::get().get<bool>( "SHOW_DATE" ) );

        // base class update
        MainWindow::_updateConfiguration();
        update();

    }

    //________________________________________________
    void ClockWidget::_toggleHourTicks( bool state )
    {
        XmlOptions::get().set<bool>( "SHOW_HOUR_TICKS", state );
        update();
    }

    //________________________________________________
    void ClockWidget::_toggleMinuteTicks( bool state )
    {
        Debug::Throw() << "Clock::ClockWidget::_toggleMinuteTicks - " << state << endl;
        XmlOptions::get().set<bool>( "SHOW_MINUTE_TICKS", state );
        update();
    }

    //________________________________________________
    void ClockWidget::_toggleSeconds( bool state )
    {

        XmlOptions::get().set<bool>( "SHOW_SECONDS", state );
        if( state ) timer_.start( 1000, this );
        update();

    }

    //________________________________________________
    void ClockWidget::_toggleDate( bool state )
    {
        XmlOptions::get().set<bool>( "SHOW_DATE", state );
        update();
    }

    //________________________________________________
    void ClockWidget::_configureAlarms()
    {

        Debug::Throw( "Clock::ClockWidget::_configureAlarms.\n" );

        // create dialog
        AlarmListDialog dialog( this, alarms_.alarms() );
        if( dialog.centerOnDesktop().exec() == QDialog::Rejected ) return;

        // update
        alarms_.setAlarms( dialog.list() );
        alarms_.write();

        // modify timer accordingly
        if( alarms_.alarms().empty() ) alarmTimer_.stop();
        else alarmTimer_.start( 1000, this );

        return;
    }

    //________________________________________________
    void ClockWidget::_adjustTime()
    {

        Debug::Throw( "Clock::ClockWidget::_adjustTime.\n" );

        const bool alwaysUseCommand( XmlOptions::get().get<bool>( "ALWAYS_USE_CMD" ) );
        QString command( XmlOptions::get().contains( "ADJUST_TIME_CMD" ) ? QString( XmlOptions::get().raw( "ADJUST_TIME_CMD" ) ).trimmed():"" );

        if( !( alwaysUseCommand && !command.isEmpty() ) )
        {

            // create dialog
            AdjustTimeDialog dialog( this );
            dialog.setCommand( command );
            dialog.setAlwaysUse( alwaysUseCommand );
            if( dialog.exec() == QDialog::Rejected ) return;

            // retrieve command
            command = dialog.command().trimmed();

            // check if valid
            if( command.isEmpty() )
            {
                InformationDialog( this, tr( "No command set to adjust time. Canceled." ) ).exec();
                return;
            }

            XmlOptions::get().set<bool>( "ALWAYS_USE_CMD", dialog.alwaysUse() );

        }

        // try run
        int exitCode( QProcess::execute( command ) );
        if( exitCode != 0 )
        {
            const QString buffer = QString( tr( "Command '%1' excited with code %2" ) ).arg( command ).arg( exitCode );
            InformationDialog( this, buffer ).exec();

            XmlOptions::get().set<bool>( "ALWAYS_USE_CMD", false );

        } else {
            XmlOptions::get().setRaw( "ADJUST_TIME_CMD", command );
            XmlOptions::write();
        }

        update();

        return;

    }


    //____________________________________________________________________
    void ClockWidget::_toggleInverseColors( bool value )
    {

        Debug::Throw( "ClockWidget::_toggleInverseColors" );
        MainWindow::_toggleInverseColors( value );

        // update internal colors
        if( XmlOptions::get().get<bool>( "USE_DEFAULT_COLORS" ) )
        {
            _setHourColor( foregroundColor() );
            _setMinuteColor( foregroundColor() );
            _setSecondColor( foregroundColor() );
        }

        update();
    }

    //__________________________________________________________________________
    void ClockWidget::_checkAlarms()
    {


        // retrieve the data and time from now
        QDateTime current( QDateTime::currentDateTime().addSecs( 3600*delay_ ) );

        QDate currentData( current.date() );
        QTime currentTime( current.time() );
        for( auto& alarm:alarms_.alarms() )
        {
            if( !alarm.triggered( currentData, currentTime ) ) continue;

            // update window with current time
            // this is needed for when the method is called prior to updating the clock display
            // in which case there appear to be a mismatch between the two.
            update();

            // create and raise alarm dialog
            AlarmDialog* dialog( new AlarmDialog( 0, alarm ) );
            connect( dialog, SIGNAL(repeat(Alarm)), SLOT(_setRepeat(Alarm)) );
            connect( dialog, SIGNAL(closed(Alarm)), SLOT(_unsetRepeat(Alarm)) );
            QtUtil::centerOnDesktop( dialog )->show();
        }

        // restart timer in a minute.
        alarmTimer_.start( 1000*(60-currentTime.second() ), this );
        update();

        return;

    }

    //_______________________________________________________
    QPainterPath ClockWidget::_hand( int width, int length )
    {
        QPainterPath out;
        out.moveTo( QPointF( -width, -length+width ) );
        out.arcTo( QRectF( -width, -length - width, 2*width, 2*width ), 180, -180 );
        out.lineTo( QPointF( width, 0 ) );
        out.arcTo( QRectF( -width, -width, 2*width, 2*width ), 0, -180 );
        out.closeSubpath();
        return out;
    }

}
