#ifndef CpuLoad_h
#define CpuLoad_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "QOrderedSet.h"

#include <QTextStream>
#include <QDateTime>
#include <QHash>
#include <QList>

namespace Cpu
{

    //* measures relative cpu load from /proc/stat file
    class CpuLoad final: private Base::Counter<CpuLoad>
    {

        public:

        //* Data type enumeration
        enum DataType
        {
            User = 1<<0,
            System = 1<<1,
            Nice = 1<<2,
            All = User|System|Nice
        };

        Q_DECLARE_FLAGS( DataTypes, DataType );

        //* constructor
        explicit CpuLoad();

        //* cpu load data
        class Data
        {

            public:

            //* constructor
            explicit Data()
            {}

            //* difference between two points
            Data& operator -= ( const Data& other )
            {
                user_ -= other.user_;
                nice_ -= other.nice_;
                system_ -= other.system_;
                total_ -= other.total_;
                time_ -= other.time_;
                valid_ &= ( other.valid_ && time_ > 0 );
                return *this;
            }

            //* load from stream
            friend QTextStream &operator >> ( QTextStream &in, Data& data )
            {
                in >> data.user_ >> data.nice_ >> data.system_;

                // idle is supposed to be the last number
                int tmp = 0;
                data.total_ = 0;
                while( in.status() == QTextStream::Ok )
                {
                    data.total_ += tmp;
                    in >> tmp;
                }

                data.valid_ = data.total_ > 0;
                data.total_ += data.user_ + data.nice_ + data.system_;

                return in;
            }

            //* user time
            int user() const
            { return user_; }

            //* nice time
            int nice() const
            { return nice_; }

            //* system time
            int system() const
            { return system_; }

            //* sum
            int sum( DataTypes type = All ) const
            {
                int out( 0 );
                if( type & User ) out += user();
                if( type & System ) out += system();
                if( type & Nice ) out += nice();
                return out;
            }

            //* idle time
            int total() const
            { return total_; }

            //* total_cpu over total (in %)
            int ratio( DataTypes type = All ) const
            { return 100*sum(type)/total_; }

            //* time
            void setTime( int time )
            { time_ = time; }

            //* time
            int time() const
            { return time_; }

            //* validity
            bool valid() const
            { return valid_; }

            //* used to sort data according to selected data type
            class LessFTor
            {

                public:

                //* constructor
                explicit LessFTor( DataTypes type ):
                    type_( type )
                {}

                //* predicate
                bool operator()( const Data& first, const Data& second ) const
                { return double(first.sum( type_ ))/first.time() < double( second.sum( type_ ))/second.time(); }

                private:

                //* data type
                DataTypes type_;

            };

            private:

            //* user time
            int user_ = 0;

            //* nice time
            int nice_ = 0;

            //* system time
            int system_ = 0;

            //* idle time
            int total_ = 0;

            //* measurement time
            int time_ = 0;

            //* true if current data are valid
            bool valid_ = false;

        };

        //* shortcut for data list
        using DataList = QList< Data >;

        //* CpuLoad data samples storage
        class Samples final: private Base::Counter<Samples>
        {

            public:

            //* construtor
            explicit Samples( int sampleSize = 0, int dataSize = 0 ):
                Counter( "CpuLoad::Samples" ),
                sampleSize_( sampleSize ),
                dataSize_( dataSize )
            { }

            //* set sample size
            void setSampleSize( int );

            //* set data point size
            void setDataSize( int );

            //* retrieve data
            const DataList& data() const
            { return dataList_; }

            //* add a data point
            void add( const Data& data );

            private:

            //* list of cpu load measurement
            DataList sampleList_;

            //* list of cpu load differences
            DataList dataList_;

            //* number of samples to be averaged to make the data points
            int sampleSize_ = 0;

            //* number of data point to store
            int dataSize_ = 0;

        };

        //* shortcut for Samples set
        using SampleMap = QHash< QString, Samples >;

        //* cpu names
        using CpuNames = QOrderedSet<QString>;

        //* get available cpu names
        CpuNames cpuNames() const;

        //* changes sampleSize_
        void setSampleSize( int );

        //* changes sampleSize_
        void setDataSize( int );

        //* retrieve data for a given cpu name
        DataList data( const QString& cpuName = "cpu" ) const;

        //* update all Samples from file
        void update();

        //* used to sort data according to selected data type
        class LessFTor
        {

            public:

            //* constructor
            explicit LessFTor( DataTypes type ):
                type_( type )
            {}

            //* predicate
            bool operator()( const Data& first, const Data& second ) const
            { return double(first.sum( type_ ))/first.time() < double( second.sum( type_ ))/second.time(); }

            private:

            //* data type
            DataTypes type_;

        };

        private:

        //* filename
        const QString filename_;

        //* number of samples to average on
        int sampleSize_ = 0;

        //* number of data to be kept for each cpu
        int dataSize_ = 0;

        //* data samples
        SampleMap sampleMap_;

        //* timer
        QTime timer_;

    };

};

Q_DECLARE_OPERATORS_FOR_FLAGS( Cpu::CpuLoad::DataTypes )

#endif
