/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConfigurationDialog.h"

#include "GridLayout.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "OptionAnchorWidget.h"
#include "OptionCheckBox.h"
#include "OptionSpinBox.h"
#include "SvgConfiguration.h"
#include "TransparencyConfiguration.h"

#include <QButtonGroup>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QPushButton>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
BaseConfigurationDialog( parent )
{

    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );

    // base configuration
    baseConfiguration( nullptr, Flag::Base|Flag::List);

    // transparency configuration
    QWidget* page;
    Transparency::TransparencyConfiguration* transparencyConfiguration;

    // generic objects
    QGroupBox *box;
    OptionCheckBox *checkbox;
    OptionSpinBox* spinbox;
    QLabel* label;

    // window
    page = &addPage( IconEngine::get( IconNames::PreferencesMainWindow ), tr( "Main Window" ), tr( "Window geometry and behavior" ) );

    // geometry
    page->layout()->addWidget( box = new QGroupBox( tr( "Geometry" ), page ));

    GridLayout* gridLayout = new GridLayout;
    gridLayout->setSpacing(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment(0, Qt::AlignVCenter|Qt::AlignRight );
    gridLayout->setColumnAlignment(1, Qt::AlignVCenter|Qt::AlignLeft );
    gridLayout->setColumnStretch(0, 0);
    gridLayout->setColumnStretch(1, 1);
    box->setLayout( gridLayout );

    gridLayout->addWidget( label = new QLabel( tr( "Width:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "WINDOW_WIDTH" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setMinimum( 5 );
    spinbox->setMaximum( 2048 );
    spinbox->setToolTip( tr( "Main window width (pixels)" ) );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    gridLayout->addWidget( label = new QLabel( tr( "Height:" ), box ) );
    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "WINDOW_HEIGHT" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setMinimum( 5 );
    spinbox->setMaximum( 2048 );
    spinbox->setToolTip( tr( "Main window height (pixels)" ) );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    gridLayout->addWidget( label = new QLabel( tr( "Margin:" ), box ) );
    label->setAlignment( Qt::AlignVCenter|Qt::AlignRight );

    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "WINDOW_BORDER" ) );
    spinbox->setSuffix( tr( "px" ) );
    spinbox->setMinimum( 0 );
    spinbox->setMaximum( 100 );
    spinbox->setToolTip( tr( "Distance (pixels) between window edge and text/histogram" ) );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    // position
    page->layout()->addWidget( box = new QGroupBox( tr( "Position" ), page ));
    box->setLayout( new QVBoxLayout );
    box->layout()->setSpacing(5);

    OptionAnchorWidget* anchor = new OptionAnchorWidget( box, "ANCHOR" );
    box->layout()->addWidget( anchor );
    addOptionWidget( anchor );

    // options
    page->layout()->addWidget( box = new QGroupBox( tr( "Options" ), page ));
    QVBoxLayout *boxLayout = new QVBoxLayout;
    boxLayout->setSpacing(5);
    box->setLayout( boxLayout );

    boxLayout->addWidget( checkbox = new OptionCheckBox( tr( "Keep above others" ), box, "STAYS_ON_TOP" ));
    checkbox->setToolTip( tr( "Keep window above all others" ) );
    addOptionWidget( checkbox );

    boxLayout->addWidget( checkbox = new OptionCheckBox( tr( "Show on all desktops" ), box, "STICKY" ));
    checkbox->setToolTip( tr( "Make window appear on all desktops" ) );
    addOptionWidget( checkbox );

    boxLayout->addWidget( checkbox = new OptionCheckBox( tr( "Hide from taskbar" ), box, "HIDE_FROM_TASKBAR" ));
    checkbox->setToolTip( tr( "Hide window from taskbar" ) );
    addOptionWidget( checkbox );

    boxLayout->addWidget( checkbox = new OptionCheckBox( tr( "Round corners" ), this, "ROUND_CORNERS" ) );
    checkbox->setToolTip( tr( "Round/square window corners" ) );
    addOptionWidget( checkbox );

    boxLayout->addWidget( checkbox = new OptionCheckBox( tr( "Lock position" ), box, "LOCK_POSITION" ));
    checkbox->setToolTip( tr( "Lock the window position on desktop" ) );
    addOptionWidget( checkbox );

    // foreground
    page = &addPage( IconEngine::get( IconNames::PreferencesColors ), tr( "Foreground" ), tr( "Foreground color settings" ) );
    page->layout()->addWidget( transparencyConfiguration = new Transparency::TransparencyConfiguration( page, Transparency::TransparencyConfiguration::Foreground ));
    addOptionWidget( transparencyConfiguration );

    // background
    page = &addPage( IconEngine::get( IconNames::PreferencesBackground ), tr( "Background" ), tr( "Window background settings" ) );

    Svg::SvgConfiguration* svgConfiguration;
    page->layout()->addWidget( svgConfiguration = new Svg::SvgConfiguration( page ) );
    addOptionWidget( svgConfiguration );

    read();

}
