/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "GeometryApplication.h"
#include "ApplicationType.h"
#include "BaseConfigurationDialog.h"
#include "GeometryCalculationObject.h"
#include "GeometryDialog.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QDesktopWidget>
#include <QTimer>

//____________________________________________
GeometryApplication::GeometryApplication( CommandLineArguments arguments ):
    BaseApplication( nullptr, arguments ),
    Counter( "GeometryApplication" )
{}

//____________________________________________
GeometryApplication::~GeometryApplication()
{

    Debug::Throw( "GeometryApplication::~GeometryApplication.\n" );
    if( mainWindow_ ) delete mainWindow_;

}

//____________________________________________
bool GeometryApplication::realizeWidget()
{
    Debug::Throw( "GeometryApplication::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    auto parser = commandLineParser( _arguments() );
    if( parser.hasFlag( "--list" ) )
    {

        _sendGeometryRequest();
        QTimer::singleShot( 1000, qApp, SLOT(quit()) );

    } else if( parser.hasFlag( "--batch" ) ) {

        _sendGeometryRequest();
        QTimer::singleShot( 1000, this, SLOT(_updateGeometry()) );

    } else {

        // top widget
        mainWindow_ = new GeometryDialog;
        connect( mainWindow_, SIGNAL(geometryRequested()), SLOT(_sendGeometryRequest()) );
        connect( mainWindow_, SIGNAL(geometryChanged(ApplicationData)), SLOT(_sendGeometry(ApplicationData)) );

        // show main window
        emit configurationChanged();
        mainWindow_->show();
        mainWindow_->reload();

    }

    return true;

}

//____________________________________________
CommandLineParser GeometryApplication::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( BaseApplication::commandLineParser() );

    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( CommandLineParser::Tag( "--list", "-l" ), tr( "list running transparent applications and geometry") );
    out.registerFlag( CommandLineParser::Tag( "--batch", "-b" ), tr( "apply saved automatic placement to running transparent applications" ) );

    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;
}


//____________________________________________
void GeometryApplication::usage() const
{
    _usage( "transparency-settings" );
    commandLineParser().usage();
    return;
}

//___________________________________________________________
void GeometryApplication::_configuration()
{
    Debug::Throw( "GeometryApplication::_configuration.\n" );

    // save configuration
    emit saveConfiguration();

    BaseConfigurationDialog dialog(0, BaseConfigurationDialog::Flag::Default );
    dialog.setWindowTitle( "Configuration - Transparency-settings" );

    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();
}

//_________________________________________________
void GeometryApplication::_updateGeometry()
{
    // get orientation and sort list
    const ApplicationData::Orientation orientation =  static_cast<ApplicationData::Orientation>(XmlOptions::get().get<int>("SETTINGS_ALIGNMENT") );
    ApplicationData::sort( applications_, orientation );

    // create calculation object
    GeometryCalculationObject calculationObject;
    calculationObject.setApplications( applications_ );

    // set desktop geometry
    auto desktop( qApp->desktop() );
    calculationObject.setDesktopGeometry( desktop->availableGeometry() );

    // do the calculation
    calculationObject.calculate();

    // send geometry
    for( auto&& application:calculationObject.applications() )
    { _sendGeometry( application ); }

    qApp->processEvents();
    qApp->quit();

}

//_________________________________________________
void GeometryApplication::_sendGeometryRequest()
{

    Debug::Throw( "GeometryApplication::_sendGeometryRequest.\n" );
    applications_.clear();
    _sendRequest( Server::ServerCommand::GeometryRequest );

}

//_________________________________________________
void GeometryApplication::_sendGeometry( ApplicationData data )
{

    Debug::Throw() << "GeometryApplication::_sendGeometry - " << data << endl;

    // send geometry
    Server::ApplicationId id( _applicationManager().id() );
    id.setName( data.name() );

    // create command
    QRect rect( data.geometry() );
    Server::ServerCommand command( id, Server::ServerCommand::Geometry );
    command.setArguments( CommandLineArguments(
    {
        "transparency ",
        QString::number( rect.left() ),
        QString::number( rect.top()),
        QString::number( rect.width() ),
        QString::number( rect.height() ),
        QString::number( data.order() )
    } ) );

    // send to server
    _applicationManager().client().sendCommand( command );

}

//_________________________________________________
void GeometryApplication::_sendRequest( Server::ServerCommand::CommandType command_type )
{

    Debug::Throw( "GeometryApplication::_sendRequest.\n" );

    // loop over application types
    for( auto&& iter = ApplicationType::names().begin(); iter != ApplicationType::names().end(); ++iter )
    {

        Debug::Throw() << "Application::_sendRequest - sending to: " << iter.value() << endl;

        // create command that match name
        Server::ApplicationId id( _applicationManager().id() );
        id.setName( iter.value() );

        // create command
        Server::ServerCommand command( id, command_type );
        _applicationManager().client().sendCommand( command );

    }

}

//________________________________________________
bool GeometryApplication::_processCommand( Server::ServerCommand command )
{


    // try base class
    if( BaseApplication::_processCommand( command ) ) return true;

    // test command type
    switch( command.command() )
    {

        case Server::ServerCommand::Raise:
        if( mainWindow_ ) mainWindow_->uniconify();
        return true;

        case Server::ServerCommand::Geometry:
        {

            // parse command
            auto arguments( command.arguments().get() );
            arguments.removeFirst();
            if( arguments.size() != 7 )
            {

                Debug::Throw(0) << "GeometryApplication::_processCommand - invalid number of arguments (" << arguments.size() << "): " << arguments.join( " " ) << endl;

            } else {

                ApplicationData data;
                data.setName( arguments[0] );
                data.setGeometry( QRect(
                    QPoint( arguments[1].toInt(), arguments[2].toInt() ),
                    QSize( arguments[3].toInt(), arguments[4].toInt() ) ) );

                data.setOrder( arguments[5].toInt() );
                data.setIcon( QIcon( arguments[6] ));

                auto parser = commandLineParser( _arguments() );
                if( parser.hasFlag( "--list" ) ) Debug::Throw(0) << data << endl;


                Debug::Throw() << "GeometryApplication::_processCommand - received " << data << endl;

                // add to main window
                if( mainWindow_ ) mainWindow_->add( data );

                // add to internal list
                applications_.append( data );

            }

        }
        return true;

        default: return false;
    }
}
