/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CustomProcess.h"
#include "Debug.h"
#include "NetworkLoad.h"
#include "NetworkConnectionMonitor.h"

#include <QFile>
#include <QStringList>

namespace Network
{

    //__________________________________
    NetworkLoad::NetworkLoad():
        Counter( "NetworkLoad" ),
        filename_( "/proc/net/dev" )
    { timer_.start(); }

    //________________________________________________
    QString NetworkLoad::toString( double value )
    {

        QString unit( QObject::tr( "bps" ) );
        if( value > 1024 ) { value/=1024; unit = QObject::tr( "kbps" ); }
        if( value > 1024 ) { value/=1024; unit = QObject::tr( "Mbps" ); }
        return QString( "%1 %2" ).arg( QString::number( value, 'f', 1 ) ).arg( unit );

    }

    //________________________________________________
    NetworkLoad::DeviceSet NetworkLoad::devices() const
    {
        DeviceSet out;
        for( auto&& iter = sampleMap_.begin(); iter != sampleMap_.end(); ++iter )
        { out.insert( iter.key() ); }
        return out;
    }

    //________________________________________________
    NetworkLoad::DeviceSet NetworkLoad::connectedDevices() const
    {
        Debug::Throw( "NetworkLoad::connectedDevices.\n" );

        DeviceSet out;
        for( auto&& device:ConnectionMonitor::connectedDevices() )
        { out.insert( device ); }

        return out;

    }

    //________________________________________________
    void NetworkLoad::update()
    {

        Debug::Throw( "NetworkLoad::update.\n" );

        // open file
        QFile in( filename_ );
        if( !in.open( QIODevice::ReadOnly ) )
        {
            Debug::Throw( "MemoryLoad::update - cannot open file.\n" );
            return;
        }

        // read lines
        QStringList lines( QString( in.readAll() ).split( "\n" ) );
        for( auto line:lines )
        {
            if( line.indexOf( ":" ) < 0 ) continue;
            line.replace( ":", " " );
            QTextStream buffer( &line, QIODevice::ReadOnly );
            Data data;
            QString device_name;
            buffer >> device_name >> data;
            data.setTime( timer_.elapsed()/1000 );

            // skip "lo" device
            if( device_name == "lo" ) continue;

            // initialize DataSamples if new
            if( sampleMap_.find( device_name ) == sampleMap_.end() ) {
                sampleMap_[ device_name ].setSampleSize( sampleSize_ );
                sampleMap_[ device_name ].setDataSize( dataSize_ );
            }

            // add current data to matching sample
            sampleMap_[ device_name ].add( data );

        }

        return;

    }

    //________________________________________________
    NetworkLoad::DataList NetworkLoad::data( const QString& device_name ) const
    {
        SampleMap::const_iterator iter( sampleMap_.find( device_name ) );
        if( iter == sampleMap_.end() ) return DataList();
        else return iter.value().data();
    }

    //________________________________________________
    void NetworkLoad::DataSamples::setSampleSize( int sampleSize )
    {
        sampleSize_ = sampleSize;
        while( sampleList_.size() > sampleSize_ )
        { sampleList_.removeFirst(); }
    }

    //________________________________________________
    void NetworkLoad::DataSamples::setDataSize( int dataSize )
    {
        dataSize_ = dataSize;
        while( dataList_.size() > dataSize_ )
        { dataList_.removeFirst(); }
    }

    //________________________________________________
    void NetworkLoad::DataSamples::add( const NetworkLoad::Data& data )
    {

        // add data to samples
        sampleList_.append( data );

        // check size
        if( sampleList_.size() > sampleSize_ ) {

            auto data( sampleList_.back() );
            data -= sampleList_.front();
            if( data.valid() ) dataList_ << data;
            if( dataList_.size() > dataSize_ ) dataList_.removeFirst();
            sampleList_.removeFirst();

        }

        return;

    }

}
