/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "PictureParser.h"

#include "Debug.h"
#include "File.h"
#include "PictureXmlDef.h"
#include "XmlDocument.h"

#include <QFile>

namespace Pictures
{

    //________________________________________________
    bool PictureParser::read( const File& file )
    {
        Debug::Throw( "PictureParser::Read.\n" );

        // reset attributes_list
        attributesList_.clear();

        if( file.localName().startsWith( '.' ) ) file.setHidden();

        // parse the file
        XmlDocument document;
        {
            QFile qfile( file );
            if( !document.setContent( &qfile ))
            {
                Debug::Throw() << document.error() << endl;
                return false;
            }
        }

        // look for relevant element
        QDomNodeList topNodes = document.elementsByTagName( Xml::PictureList );
        if( topNodes.isEmpty() ) return false;
        for(QDomNode node = topNodes.at(0).firstChild(); !node.isNull(); node = node.nextSibling() )
        {
            QDomElement element = node.toElement();
            if( element.isNull() ) continue;

            QString tag_name( element.tagName() );
            if( tag_name == Xml::Picture ) attributesList_ << PictureAttributes( element );
            else Debug::Throw(0) << "PictureParser::Read - unrecognized tag_name " << tag_name << endl;
        }

        return true;
    }

    //________________________________________________
    bool PictureParser::write( const File& file ) const
    {

        Debug::Throw() << "PictureParser::Write - file: " << file << endl;

        // create document and read
        XmlDocument document;
        {
            QFile qtfile( file );
            document.setContent( &qtfile );
        }

        // create main element and insert records
        QDomElement top = document.createElement( Xml::PictureList );
        for( const auto& attributes:attributesList_ )
        { top.appendChild( attributes.domElement( document.get() ) ); }

        // append top node to document and write
        document.replaceChild( top );
        {
            QFile qfile( file );
            if( !qfile.open( QIODevice::WriteOnly ) ) return false;
            qfile.write( document.toByteArray() );
        }

        return true;

    }

}
